/* -*- mode: c++ -*- */
// interpol.h
// Algoritmo para interpolar un polinomio a un conjunto de nodos

#ifndef _INTERPOL___
#define _INTERPOL___

#include <algoritmo.h>
#include <cdatos.h>
#include <dvector.h>

using namespace std;

/**
 *\defgroup P6 Documentacin de la prctica 6 (Interpolacin)
 */

/// Mquina de interpolar
/*
 *\ingroup P6
 */

class Interpol: public Algoritmo {
private:
/// abcisas
    DVector pX;   
/// ordenadas
    DVector pY;   
///  coeficientes 
    DVector pC;   
/// grado del polinomio
    int pGrado;   
/// extremo inferior polinomio
    int pInf;     
/// extremo superior polinomio
    int pSup;     

  /// Calcula coeficientes del polinomio interpolador
  /**
   * Los coeficientese se calculan 
   * en el rango [inf,sup], mediante diferencias divididas
   */
    void calculaCoeficientes(const DVector& x, DVector& c, 
			     int inf, int sup);
public:
    /// Constructor
    Interpol();
  /// Destructor
    ~Interpol();

    // Funciones de acceso

    /// devuelve el grado del polinomio interpolado
    int grado() const;  
    // devuelve los datos
    /// abcisas
    const DVector& x() const;
  /// ordenadas
    const DVector& fx() const;
  /// devuelve el vector de coeficientes actualmente interpolado
    const DVector& ci() const;

    /// Establece datos
    void eDatos(const CDatos& datos);
  /// Establece polinomio
  /** 
   *interpolamos un polinomio a los puntos en el intervalo [i,j]
   */
    void ePolinomio(int i, int j);

    // Acciones
    /// valor del polinomio actualmente interpolado en x
  double p(double x);
};


#endif   /* _INTERPOL___ */
