#include <iostream>
#include <iomanip>
#include <interpol.h>
#include <string>
#include <error.h>
#include <matutil.h>

const double npuntos = 50;

inline double sin3(double x)
{
    return cubo(sin(x));
}
inline double runge(double x)
{
    return 1./(1+cuadrado(x));
}
inline double ff(double x)
{
    return atan(x);
}

int main ()
{
    // Lee datos a interpolar 
    string fdatos;
    cout << "fichero de datos a interpolar? "; cin >> fdatos; 
    cout << " Qu funcin desea? (1 = sin**3(x), 2 = runge, 3 = F)";
    int ifun; cin >> ifun;

    CDatos cd;
    cd.lee(fdatos);

    // Vectores de abcisas y ordenadas
    DVector x= cd.x();
    DVector y= cd.y();

    // Crea un interpolador
    Interpol xpol;
    // Establece datos
    xpol.eDatos(cd);
    // Ajusta un polinomio de grado n 
    // entre los nodos imin e imax

    cout << " Este programa interpola un polinomio"
	 <<" a un conjunto de "<<cd.dimension()<<" nodos obtenidos a partir"
	 <<" de la funcin representada en el fichero " << fdatos 
	 << endl;
    cout << "El polinomio se interpola entre los nodos"
	 << " [imin, imax], donde imin >= 0 y imax <= "<<cd.dimension()-1
	 << endl;

    int imin, imax;
    cout << "imin? "; cin >> imin;
    cout << "imax? "; cin >> imax;

    cout << "-------------------" << endl;

    cout << "nodo inicial xi = " << imin 
	 << " x["<<imin<<"]=" << x[imin] 
	 << " f(x["<<imin<<"])=" << y[imin] 
	 << endl;
    cout << "nodo final xf = " << imax 
	 << " x["<<imax<<"]=" << x[imax] 
	 << " f(x["<<imax<<"])=" << y[imax] 
	 << endl;


    // establece el polinomio
    xpol.ePolinomio(imin,imax);
    int n = imax - imin; // grado del polinomio

    cout << " nmero de nodos a interpolar " << imax - imin + 1 << endl;
    cout << " grado del polinomio interpolador " << n << endl;
 
    // fichero de salida con una tabla de valores interpolados
    // que no coinciden con los nodos
    string fout = "p"+fdatos;

    ofstream fcout(fout.c_str(),ios::out);
    if ( !fcout)
	error("Imposible abrir fichero en CDatos::xinterpol",
	      CENOFCHRO);

    cout << " Calcula tabla de valores entre "
	 <<x[imin] <<" y " 
	 <<x[imax] <<endl;
 
    double step = (x[imax]- x[imin])/(npuntos);
    double xx = x[imin]+ step;
    cout << "\t x \t\t f(x) \t p(x)\t \t |p(x)-f(x)|" << endl;
    cout << "-------------------------------------------------------------------" 
	 << endl;
    
    while (xx < x[imax]){
	double dd; 
	if (ifun == 1) 
	    dd = abs(xpol.p(xx)-sin3(xx))  ;
	else  if (ifun == 2) 
	    dd = abs(xpol.p(xx)-runge(xx))  ;
	else  if (ifun == 3) 
	    dd = abs(xpol.p(xx)-ff(xx))  ;
	cout << xx << "\t" ;
	if (ifun == 1) {
	    cout << setw(12) << setprecision(10)
		 << sin3(xx) << "\t";
	}
	else  if (ifun == 2) {
	    cout << setw(12) << setprecision(10)
		 << runge(xx) << "\t";

	}	
	else  if (ifun == 3) {
	    cout << setw(12) << setprecision(10)
		 << ff(xx) << "\t";

	}
	
	cout << setw(12) << setprecision(10) 
	     << xpol.p(xx) << "\t"
	     << setw(12) << setprecision(10) 
	     << dd
	     << endl;

	fcout << xx << "\t" 
	      << xpol.p(xx) << endl;
	xx += step;
    }
    
    return 0;
}
